# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django import forms
from django.forms.formsets import formset_factory
from django.forms import IntegerField
from django.forms import DateField
from django.forms import DateTimeField
from django.forms import CharField
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _
from django.utils import timezone
from project.forms.fields.checkbox import CheckboxField
from project.forms.fields.text import TextField
from project.forms.fields.extras.datetime import SelectDateTimeField
from project.view import FuncView
from project.fal.syslogsrv import FALSyslogSrv, FACILITIES, PRIORITIES
from datetime import datetime, timedelta, tzinfo
from logfilter import LogFilter
import os
import settings

CODEC_UTF8 = "utf8"
CODEC_SHIFT_JIS = "shift_jis"
CODEC_EUC_JP = "euc_jp"
REPLACE_ARG = "replace"

TARGET_CODEC_LIST = [CODEC_UTF8, CODEC_SHIFT_JIS, CODEC_EUC_JP]


def _to_unicode_message(message_orig):

    for codec in TARGET_CODEC_LIST:
        try:
            return message_orig.decode(codec)
        except:
            pass

    return message_orig.decode(CODEC_UTF8, REPLACE_ARG)


class ViewerForm(forms.Form):

    def __init__(self, *args, **kwargs):
        super(ViewerForm, self).__init__(*args, **kwargs)

        self.fields['datetime_from'] = SelectDateTimeField(
                label="%s %s" % (_(u'DateTime'), _(u'Timestamp(from)')),
                required=False)
        self.fields['datetime_to'] = SelectDateTimeField(
                label="%s %s" % (_(u'DateTime'), _(u'Timestamp(to)')),
                required=False)
        self.fields['hostname'] = TextField(
                label=_(u"Hostname"),
                attrs={'class': 'hostname'},
                max_length=1024,
                required=False)
        self.fields['keyword'] = TextField(
                label=_(u"Keyword"),
                attrs={'class': 'keyword'},
                max_length=1024,
                required=False)

        # priotities
        for i, pri in enumerate(LogFilter.PRIORITIES):
            self.fields['pri%d' % i] = CheckboxField(label=pri)
        self.fields['prix'] = CheckboxField(label=_(u"others"))

        # facilities
        for i, fac in enumerate(LogFilter.FACILITIES):
            self.fields['fac%d' % i] = CheckboxField(label=fac)
        self.fields['facxx'] = CheckboxField(label=_(u"others"))

        # 使用する優先度リストを定義する その他(other) は99
        self.priorities = [0,1,2,3,4,5,6,7]

        # 使用するファシリティリストを定義する その他(other) は99
        self.facilities = [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,
                            20,21,22,23,99]

    def clean(self):
        f = self.cleaned_data.get('datetime_from', None)
        t = self.cleaned_data.get('datetime_to', None)
        if isinstance(f, datetime) and isinstance(t, datetime) and f > t:
            raise forms.ValidationError(_(u'"To DateTime" before "From DateTime"'))
        return self.cleaned_data

class ViewerView(FuncView):

    views = ['apply']

    def view(self, request):
        logs = []
        form = ViewerForm(initial=self.__initial())
        template = os.path.join(self.TEMPLATES, 'template.html')

        dictionary = {
            'form': form,
            'pris': form.priorities,
            'facs': form.facilities,
            'logs': logs,
            'nums': 0,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_apply(self, request):
        logs = []
        form = ViewerForm(data=request.POST)

        with self.processing(constname="ERR_SYSLOG_VIEWER") as p:
            if form.is_valid():
                lf = LogFilter()
                pri = 0
                mask = 0
                for v in form.priorities:
                    if v == 99:
                        pri = ((pri | ~mask) & 0xFF) | lf.BIT_OTHERS
                    elif v < len(LogFilter.PRIORITIES):
                        cb = form.cleaned_data['pri%d' % v]
                        pri = pri | (1 << v) if cb == True else pri
                        mask = mask | (1 << v)
                fac = 0
                mask = 0
                for v in form.facilities:
                    if v == 99:
                        fac = ((fac | ~mask) & ((1 << 24) - 1)) | lf.BIT_OTHERS
                    elif v < len(LogFilter.FACILITIES):
                        cb = form.cleaned_data['fac%d' % v]
                        fac = fac | (1 << v) if cb == True else fac
                        mask = mask | (1 << v)
                lines = lf.collect({
                    'datetime_from': form.cleaned_data['datetime_from'],
                    'datetime_to'  : form.cleaned_data['datetime_to'],
                    'hostname'     : form.cleaned_data['hostname'],
                    'keyword'      : form.cleaned_data['keyword'],
                    'priority_mask': pri,
                    'facility_mask': fac})
                if len(lines) == 0:
                    self.set_message(_(u"log not found"))
                for l in lines:
                    t = l.split(' ', 5)
                    if len(t) < 6:
                        continue

                    message = _to_unicode_message(t[5])

                    logs.append({
                        'datetime': lf.str_to_datetime(t[0]),
                        'hostname': t[1],
                        'facility': t[3],
                        'priority': t[4],
                        'message' : message})
                p.result(True)
            else:
                p.result(False)

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'form': form,
            'pris': form.priorities,
            'facs': form.facilities,
            'logs': logs,
            'nums': len(logs),
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def __initial(self):
        try:
            return {
                'hostname': "",
                #'datetime_from': datetime.now() - timedelta(days=7),
                'pri0': True, 'pri1': True, 'pri2': True, 'pri3': True,
                'pri4': True, 'pri5': True, 'pri6': True, 'pri7': True,
                'prix': False,
                'fac0' : True, 'fac1' : True, 'fac2' : True, 'fac3' : True,
                'fac4' : True, 'fac5' : True, 'fac6' : True, 'fac7' : True,
                'fac8' : True, 'fac9' : True, 'fac10': True, 'fac11': True,
                'fac12': True, 'fac13': True, 'fac14': True, 'fac15': True,
                'fac16': True, 'fac17': True, 'fac18': True, 'fac19': True,
                'fac20': True, 'fac21': True, 'fac22': True, 'fac23': True,
                'facxx': True,
            }
        except:
            return {}
